/****************************************************************************
 * This file is part of Fluid.
 *
 * Copyright (C) 2013 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
 *
 * Author(s):
 *    Pier Luigi Fiorini
 *
 * $BEGIN_LICENSE:LGPL2.1+$
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * $END_LICENSE$
 ***************************************************************************/

import QtQuick 2.1

/*!
    \qmltype Icon
    \inqmlmodule FluidUi 0.2
    \ingroup fluid
    \brief Displays an icon from the icon theme or another location.

    An icon from the icon theme can be specified with the \c iconName property
    otherwise an icon from a local or remote source can be specified with the
    \c iconSource property.  Icons can also be colorized, setting the \c color
    property will make all pixels with the \c keyColor colored.

    Icon themes are a set of icons referred to by a standard name defined in the
    \l{http://standards.freedesktop.org/icon-naming-spec/icon-naming-spec-latest.html}{Freedesktop Icon Naming Specification}
    to improve consistency between applications.

    Icon themes also respect the
    \l{http://standards.freedesktop.org/icon-theme-spec/icon-theme-spec-latest.html}{Freedesktop Icon Theme Specification}.

    Example of icon from the icon theme:
    \code
    Icon {
        iconName: "gimp"
        width: 64
        height: 64
    }
    \endcode

    Example of icon from a local source:
    \code
    Icon {
        iconSource: "edit-cut.png"
        width: 16
        height: 16
    }
    \endcode

    Example of colorization:
    \code
    Icon {
        iconName: "edit-cut-symbolic"
        width: 16
        height: 16
        color: "green"
    }
    \endcode
*/
Item {
    id: icon

    /*! Name of the icon to display. */
    property string iconName

    /*! Source of the icon to display. */
    property url iconSource

    /*! Color of pixels that originally were of \c keyColor color. */
    property alias color: colorizedImage.keyColorOut

    /*! Color of the pixels that should be colorized.

        This is set to \l #bebebe by default.
    */
    property alias keyColor: colorizedImage.keyColorIn

    Image {
        id: image

        property bool ready: false

        anchors.fill: parent
        source: {
            // Don't load icons if size is not set or we don't know neither a source nor a name
            if (ready && width > 0 && height > 0 && (icon.iconName || icon.iconSource)) {
                if (icon.iconSource != "")
                    return icon.iconSource;
                return icon.iconName ? "image://desktoptheme/" + icon.iconName : "";
            }
            return "";
        }
        sourceSize {
            width: width
            height: height
        }
        cache: true
        smooth: true
        visible: !colorizedImage.active

        Component.onCompleted: ready = true
    }

    ShaderEffect {
        id: colorizedImage

        property bool active: keyColorOut != Qt.rgba(0, 0, 0, 0)
        property Image source: visible ? image : null
        property color keyColorIn: "#bebebe"
        property color keyColorOut: Qt.rgba(0, 0, 0, 0)
        property real threshold: 0.1

        anchors.fill: parent
        fragmentShader: "
            varying highp vec2 qt_TexCoord0;
            uniform sampler2D source;
            uniform highp vec4 keyColorIn;
            uniform highp vec4 keyColorOut;
            uniform lowp float threshold;
            uniform lowp float qt_Opacity;

            void main() {
                lowp vec4 sourceColor = texture2D(source, qt_TexCoord0);
                gl_FragColor = mix(vec4(keyColorOut.rgb, 1.0) * sourceColor.a, sourceColor, step(threshold, distance(sourceColor.rgb / sourceColor.a, keyColorIn.rgb))) * qt_Opacity;
            }
        "
        visible: active && image.status == Image.Ready
    }
}
